#ifndef LUM_TAB_H
#define LUM_TAB_H

/*
  This source is part of the Illumination library
  Copyright (C) 2004  Tim Teulings

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include <list>

#include <Lum/Base/Size.h>

#include <Lum/Model/Integer.h>

#include <Lum/Object.h>

namespace Lum {

  class LUMAPI Tab : public Control
  {
  private:
    class Entry
    {
    public:
      Object *rider;
      Object *object;
    };

  public:
    class Prefs : public Object::Prefs
    {
    public:
      OS::FillRef  riderBackground;   //! background of rider
      OS::FrameRef tabFrame;          //! frame around the object area
      OS::FrameRef riderFrame;        //! frame of a single rider
      bool         hilightSelection;  //! highlight selected tab rider

    public:
      Prefs();
      void Initialize();
    };

  private:
    Model::SizeTRef  model;      //! numeric model defining the currently visible entry.
    size_t           mw;         //! width of single tab rider
    size_t           mh;         //! height of single tab rider
    Object           *lastDrawn; //! At that time visible object during last draw

    std::list<Entry> list;       //! list of embeded objects

  private:
    Object* GetObject() const;

  public:
    Tab();
    ~Tab();

    bool VisitChildren(Visitor &visitor, bool onlyVisible);

    void Add(Object* label, Object* object);
    void Add(const std::wstring& label, Object* object);

    void ActivatePreviousTab();
    void ActivateNextTab();

    bool SetModel(Base::Model* model);

    void CalcSize();

    void Layout();
    void Draw(int x, int y, size_t w, size_t h);
    void Hide();

    void Resync(Base::Model* model, const Base::ResyncMsg& msg);
  };
}

#endif
