/*
  This source is part of the Illumination library
  Copyright (C) 2005  Tim Teulings

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include <Lum/TimeSelect.h>

#include <Lum/Base/L10N.h>

#include <Lum/Button.h>
#include <Lum/ButtonRow.h>
#include <Lum/Panel.h>
#include <Lum/PopupGroup.h>
#include <Lum/Space.h>
#include <Lum/Time.h>
#include <Lum/TimeView.h>

namespace Lum {
  static TimeSelect::Prefs *prefs= new TimeSelect::Prefs();

  TimeSelect::Prefs::Prefs()
  : PopupControl::Prefs(L"TimeSelect")
  {
    // no code
  }

  TimeSelect::TimeSelect()
  : PopupControl(new TimeView()),
    format(Base::timeFormatHM),
    popup(NULL)
  {
    SetPrefs(Lum::prefs);

    SetCanFocus(true);
  }

  TimeSelect::~TimeSelect()
  {
    // no code
  }

  TimeSelect::Popup::Popup(TimeSelect *select, Base::TimeFormat format)
  : format(format),
    select(select),
    commit(new Model::Action())
  {
    AttachModel(commit);
  }

  TimeSelect::Popup::~Popup()
  {
    UnattachModel(commit);
  }

  void TimeSelect::SetFormat(Base::TimeFormat format)
  {
    assert(!inited);

    this->format=format;
  }

  void TimeSelect::Popup::PreInit()
  {
    Button     *button;
    ButtonRow  *row;
    Panel      *panel;
    PopupGroup *container;
    Time       *picker;

    container=new PopupGroup();

    panel=new VPanel();

    picker=new Time();
    picker->SetFormat(format);
    picker->SetFlex(true,true);
    picker->SetModel(select->GetModel());

    panel->Add(picker);

    panel->Add(new VSpace());

    row=new ButtonRow();
    row->SetFlex(true,false);

    button=new Button();
    button->SetFlex(true,true);
    button->SetText(_ld(dlgButtonOk));
    button->SetModel(commit);
    row->Add(button);

    panel->Add(row);

    container->SetMain(panel,false);

    SetTop(container);

    Dialog::PreInit();
  }

  void TimeSelect::Popup::Resync(Base::Model* model, const Base::ResyncMsg& msg)
  {
    if (model==commit && commit->IsFinished()) {
      select->GetModel()->Save();
      Exit();
    }

    Dialog::Resync(model,msg);
  }

  void TimeSelect::OpenPopup()
  {
    GetModel()->Push();

    popup=new Popup(this,format);
    popup->SetType(OS::Window::typePopup);
    popup->SetParent(GetWindow());
    popup->SetReference(this);

    if (popup->Open()) {
      popup->EventLoop();
      popup->Close();
    }

    delete popup;
    popup=NULL;

    GetModel()->Pop();
  }

  void TimeSelect::Resync(Base::Model* model, const Base::ResyncMsg& msg)
  {
    if (model==popupAction && popupAction->IsFinished()) {
      OpenPopup();
    }

    PopupControl::Resync(model,msg);
  }

}

